package org.zheng.demo;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Optional;
import java.util.Random;

import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.zheng.demo.entity.Customer;
import org.zheng.demo.entity.Order;
import org.zheng.demo.repo.CustomerRepo;
import org.zheng.demo.repo.OrderRepo;
import org.zheng.demo.service.MajorCustomerService;
import org.zheng.demo.service.CustomerOrderService;

@SpringBootApplication
public class SebselectDemoApplication {

	public static void main(String[] args) {
		SpringApplication.run(SebselectDemoApplication.class, args);

	}

	@Bean
	public CommandLineRunner demo(CustomerRepo custRep, OrderRepo orderRep, CustomerOrderService orService,
			MajorCustomerService ms) {
		return (args) -> {
			// save a few customers
			String[] custs = { "mary", "zheng", "john", "bob", "terry" };
			String[] orders = { "PC", "Phone", "Laptop", "TV", "XBox" };
			BigDecimal[] amount = { new BigDecimal(1000), new BigDecimal(500), new BigDecimal(800),
					new BigDecimal(3000), new BigDecimal(600) };

			Arrays.asList(custs).stream().forEach(cust -> {
				Customer customer = new Customer(cust, cust + "@test.com");
				custRep.save(customer);

				Random random = new Random();
				int randomNumber = random.nextInt(orders.length);
				Order order = new Order(orders[randomNumber], randomNumber, randomNumber == 0 ? amount[randomNumber]
						: amount[randomNumber].multiply(new BigDecimal(randomNumber)), customer);
				orderRep.save(order);

				randomNumber = random.nextInt(orders.length);
				order = new Order(orders[randomNumber], randomNumber, randomNumber == 0 ? amount[randomNumber]
						: amount[randomNumber].multiply(new BigDecimal(randomNumber)), customer);
				orderRep.save(order);

			});

			orderRep.findAll().forEach(order -> {
				System.out.println(order.toString());
			});

			orService.getCustomerOrders().stream().filter(orRe -> orRe.getTotalOrderAmount().intValue() > 5000)
					.forEach(major -> {

						Optional<Customer> majorCust = custRep.findById(major.getId());

						if (majorCust.isPresent()) {
							Customer major2 = majorCust.get();
							major2.setType("Major");
							custRep.save(major2);
						}
					});

			System.out.println("Major customers with total exceed 5000.");
			ms.getMajorCustomers().stream().forEach(m -> System.out.println(m.getName()));
		};
	}

}
